from . import (
    pmxanalyse,
    pmxatom_mapping,
    pmxcreate_top,
    pmxgentop,
    pmxligand_hybrid,
    pmxmerge_ff,
    pmxmutate,
)

name = "pmxbiobb"
__all__ = [
    "pmxanalyse",
    "pmxgentop",
    "pmxmutate",
    "pmxatom_mapping",
    "pmxligand_hybrid",
    "pmxmerge_ff",
    "pmxcreate_top",
]



#!/usr/bin/env python3

"""Module containing the PMX create_top class and the command line interface."""

import argparse
import os
import shutil
import sys
from pathlib import Path, PurePath
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger


class Pmxcreate_top(BiobbObject):
    """
    | biobb_pmx Pmxcreate_top
    | Wrapper class for the `PMX create_top <https://github.com/deGrootLab/pmx>`_ module.
    | Create a complete ligand topology file from two input topology files.

    Args:
        input_topology1_path (str): Path to the input topology file 1. File type: input. `Sample file <https://github.com/bioexcel/biobb_pmx/raw/master/biobb_pmx/test/data/pmx/ligand.itp>`_. Accepted formats: itp (edam:format_3883).
        input_topology2_path (str): Path to the input topology file 2. File type: input. `Sample file <https://github.com/bioexcel/biobb_pmx/raw/master/biobb_pmx/test/data/pmx/ligand.itp>`_. Accepted formats: itp (edam:format_3883).
        output_topology_path (str): Path to the complete ligand topology file. File type: output. `Sample file <https://github.com/bioexcel/biobb_pmx/raw/master/biobb_pmx/test/data/pmx/ligand_top.zip>`_. Accepted formats: zip (edam:format_3987).

        properties (dic):
            * **force_field** (*str*) - ("amber99sb-star-ildn-mut.ff") Force-field to be included in the generated topology.
            * **water** (*str*) - ("tip3p") Water model to be included in the generated topology.
            * **system_name** (*str*) - ("Pmx topology") System name to be included in the generated topology.
            * **mols** (*list*) - ([['Protein',1],['Ligand',1]]) Molecules to be included in the generated topology.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None)  Path to the binary executable of your container.
            * **container_image** (*str*) - (None) Container Image identifier.
            * **container_volume_path** (*str*) - ("/inout") Path to an internal directory in the container.
            * **container_working_dir** (*str*) - (None) Path to the internal CWD in the container.
            * **container_user_id** (*str*) - (None) User number id to be mapped inside the container.
            * **container_shell_path** (*str*) - ("/bin/bash") Path to the binary executable of the container shell.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_pmx.pmxbiobb.pmxcreate_top import pmxcreate_top
            prop = {
                'remove_tmp' : True
            }
            pmxcreate_top(input_topology1_path='/path/to/myTopology1.itp',
                    input_topology2_path='/path/to/myTopology2.itp',
                    output_topology_path='/path/to/myMergedTopology.zip',
                    properties=prop)

    Info:
        * wrapped_software:
            * name: PMX create_top
            * version: >=1.0.1
            * license: GNU
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(
        self,
        input_topology1_path: str,
        input_topology2_path: str,
        output_topology_path: str,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {
            "in": {
                "input_topology1_path": input_topology1_path,
                "input_topology2_path": input_topology2_path,
            },
            "out": {"output_topology_path": output_topology_path},
        }

        # Properties specific for BB
        self.force_field = properties.get("force_field", "amber99sb-star-ildn-mut.ff")
        self.water = properties.get("water", "tip3p")
        self.system_name = properties.get("system_name", "Pmx topology")
        self.mols = properties.get("mols", [["Protein", 1], ["Ligand", 1]])

        # Properties common in all PMX BB
        self.gmx_lib = properties.get("gmx_lib", None)
        if not self.gmx_lib and os.environ.get("CONDA_PREFIX", ""):
            python_version = f"{sys.version_info.major}.{sys.version_info.minor}"
            self.gmx_lib = str(
                Path(os.environ.get("CONDA_PREFIX", "")).joinpath(
                    f"lib/python{python_version}/site-packages/pmx/data/mutff/"
                )
            )
            if properties.get("container_path"):
                self.gmx_lib = str(
                    Path("/usr/local/").joinpath(
                        "lib/python3.7/site-packages/pmx/data/mutff/"
                    )
                )
        self.binary_path = properties.get("binary_path", "pmx")

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Pmxcreate_top <pmx.pmxcreate_top.Pmxcreate_top>` pmx.pmxcreate_top.Pmxcreate_top object."""
        #        os.chdir("/Users/hospital/BioBB/Notebooks_dev/biobb_wf_pmx_tutorial_ligands/biobb_wf_pmx_tutorial/notebooks")
        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        fu.log("Running create_top from pmx package...\n", self.out_log)

        # Creating temporary folder
        self.tmp_folder = fu.create_unique_dir()
        fu.log("Creating %s temporary folder" % self.tmp_folder, self.out_log)

        itp = os.path.basename(
            os.path.normpath(self.stage_io_dict["in"]["input_topology1_path"])
        )
        fu.log("Creating %s itp file in temporary folder" % itp, self.out_log)
        itp_local = str(PurePath(self.tmp_folder).joinpath(itp))
        shutil.copyfile(self.io_dict["in"]["input_topology1_path"], itp_local)

        itp2 = os.path.basename(
            os.path.normpath(self.stage_io_dict["in"]["input_topology2_path"])
        )
        fu.log("Creating %s itp file in temporary folder" % itp2, self.out_log)
        itp2_local = str(PurePath(self.tmp_folder).joinpath(itp2))
        shutil.copyfile(self.io_dict["in"]["input_topology2_path"], itp2_local)

        top_local = str(PurePath(self.tmp_folder).joinpath("topology.top"))

        # _create_top function, taken from the pmx AZ tutorial:
        # https://github.com/deGrootLab/pmx/blob/develop/tutorials/AZtutorial.py
        fp = open(top_local, "w")
        # BioBB signature
        fp.write("; Topology generated by the BioBB pmxcreate_top building block\n\n")
        # ff itp
        fp.write('#include "%s/forcefield.itp"\n\n' % self.force_field)
        # additional itp
        # for i in self.itps:
        #    fp.write('#include "%s"\n' % i)
        fp.write('#include "%s"\n' % itp)
        fp.write('#include "%s"\n\n' % itp2)
        # water itp
        fp.write('#include "%s/%s.itp"\n' % (self.force_field, self.water))
        # ions
        fp.write('#include "%s/ions.itp"\n\n' % self.force_field)
        # system
        fp.write("[ system ]\n")
        fp.write("{0}\n\n".format(self.system_name))
        # molecules
        fp.write("[ molecules ]\n")
        for mol in self.mols:
            fp.write("%s %s\n" % (mol[0], mol[1]))
        fp.close()

        # zip topology
        current_cwd = os.getcwd()
        top_final = str(
            PurePath(current_cwd).joinpath(self.io_dict["out"]["output_topology_path"])
        )

        os.chdir(self.tmp_folder)
        fu.log("Compressing topology to: %s" % top_final, self.out_log, self.global_log)
        fu.zip_top(zip_file=top_final, top_file="topology.top", out_log=self.out_log)
        os.chdir(current_cwd)

        fu.log("Exit code 0\n", self.out_log)

        # Run Biobb block
        # self.run_biobb()

        # Copy files to host
        self.copy_to_host()

        self.tmp_files.append(self.stage_io_dict.get("unique_dir", ""))
        self.remove_tmp_files()

        self.check_arguments(output_files_created=True, raise_exception=False)
        return self.return_code


def pmxcreate_top(
    input_topology1_path: str,
    input_topology2_path: str,
    output_topology_path: str,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Execute the :class:`Pmxcreate_top <pmx.pmxcreate_top.Pmxcreate_top>` class and
    execute the :meth:`launch() <pmx.pmxmcreate_top.Pmxmcreate_top.launch> method."""

    return Pmxcreate_top(
        input_topology1_path=input_topology1_path,
        input_topology2_path=input_topology2_path,
        output_topology_path=output_topology_path,
        properties=properties,
        **kwargs,
    ).launch()


def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(
        description="Run PMX create_top module",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999),
    )
    parser.add_argument(
        "-c",
        "--config",
        required=False,
        help="This file can be a YAML file, JSON file or JSON string",
    )

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument(
        "--input_topology1_path",
        required=True,
        help="Path to the input topology file 1",
    )
    required_args.add_argument(
        "--input_topology2_path",
        required=True,
        help="Path to the input topology file 2",
    )
    required_args.add_argument(
        "--output_topology_path",
        required=True,
        help="Path to the complete ligand topology file",
    )

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    pmxcreate_top(
        input_topology1_path=args.input_topology1_path,
        input_topology2_path=args.input_topology2_path,
        output_topology_path=args.output_topology_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""Module containing the PMX merge_ff class and the command line interface."""

import argparse
import glob
import os
import sys
from pathlib import Path
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from pmx import ligand_alchemy  # type: ignore


class Pmxmerge_ff(BiobbObject):
    """
    | biobb_pmx Pmxmerge_ff
    | Wrapper class for the `PMX merge_ff <https://github.com/deGrootLab/pmx>`_ module.
    | Merge ligand topology files.

    Args:
        input_topology_path (str): Path to the input ligand topologies as a zip file containing a list of itp files. File type: input. `Sample file <https://github.com/bioexcel/biobb_pmx/raw/master/biobb_pmx/test/data/pmx/ligand_itps.zip>`_. Accepted formats: zip (edam:format_3987).
        output_topology_path (str): Path to the merged ligand topology file. File type: output. `Sample file <https://github.com/bioexcel/biobb_pmx/raw/master/biobb_pmx/test/data/pmx/ligand.itp>`_. Accepted formats: itp (edam:format_3883).

        properties (dic):
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.
            * **container_path** (*str*) - (None)  Path to the binary executable of your container.
            * **container_image** (*str*) - (None) Container Image identifier.
            * **container_volume_path** (*str*) - ("/inout") Path to an internal directory in the container.
            * **container_working_dir** (*str*) - (None) Path to the internal CWD in the container.
            * **container_user_id** (*str*) - (None) User number id to be mapped inside the container.
            * **container_shell_path** (*str*) - ("/bin/bash") Path to the binary executable of the container shell.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_pmx.pmxbiobb.pmxmerge_ff import pmxmerge_ff
            prop = {
                'remove_tmp' : True
            }
            pmxmerge_ff(input_topology_path='/path/to/myTopologies.zip',
                    output_topology_path='/path/to/myMergedTopology.itp',
                    properties=prop)

    Info:
        * wrapped_software:
            * name: PMX merge_ff
            * version: >=1.0.1
            * license: GNU
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(
        self,
        input_topology_path: str,
        output_topology_path: str,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {
            "in": {"input_topology_path": input_topology_path},
            "out": {"output_topology_path": output_topology_path},
        }

        # Properties specific for BB
        # None yet

        # Properties common in all PMX BB
        self.gmx_lib = properties.get("gmx_lib", None)
        if not self.gmx_lib and os.environ.get("CONDA_PREFIX", ""):
            python_version = f"{sys.version_info.major}.{sys.version_info.minor}"
            self.gmx_lib = str(
                Path(os.environ.get("CONDA_PREFIX", "")).joinpath(
                    f"lib/python{python_version}/site-packages/pmx/data/mutff/"
                )
            )
            if properties.get("container_path"):
                self.gmx_lib = str(
                    Path("/usr/local/").joinpath(
                        "lib/python3.7/site-packages/pmx/data/mutff/"
                    )
                )
        self.binary_path = properties.get("binary_path", "pmx")

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Pmxmerge_ff <pmx.pmxmerge_ff.Pmxmerge_ff>` pmx.pmxmerge_ff.Pmxmerge_ff object."""

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # Creating temporary folder
        self.tmp_folder = fu.create_unique_dir()
        fu.log("Creating %s temporary folder" % self.tmp_folder, self.out_log)

        fu.unzip_list(
            self.stage_io_dict["in"]["input_topology_path"],
            self.tmp_folder,
            out_log=self.out_log,
        )
        files = glob.glob(self.tmp_folder + "/*.itp")
        ffsIn_list = []
        for itp in files:
            ffsIn_list.append(itp)

        fu.log("Running merge_FF_files from pmx package...\n", self.out_log)
        ligand_alchemy._merge_FF_files(
            self.stage_io_dict["out"]["output_topology_path"], ffsIn=ffsIn_list
        )
        # ffsIn=[self.stage_io_dict["in"]["input_topology1_path"],self.stage_io_dict["in"]["input_topology2_path"]] )

        fu.log("Exit code 0\n", self.out_log)

        if self.gmx_lib:
            self.env_vars_dict["GMXLIB"] = self.gmx_lib

        # Run Biobb block
        # self.run_biobb()

        # Copy files to host
        self.copy_to_host()

        self.tmp_files.append(self.stage_io_dict.get("unique_dir", ""))
        self.remove_tmp_files()

        self.check_arguments(output_files_created=True, raise_exception=False)
        return self.return_code


def pmxmerge_ff(
    input_topology_path: str,
    output_topology_path: str,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Execute the :class:`Pmxmerge_ff <pmx.pmxmerge_ff.Pmxmerge_ff>` class and
    execute the :meth:`launch() <pmx.pmxmerge_ff.Pmxmerge_ff.launch> method."""

    return Pmxmerge_ff(
        input_topology_path=input_topology_path,
        output_topology_path=output_topology_path,
        properties=properties,
        **kwargs,
    ).launch()


def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(
        description="Run PMX merge_ff module",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999),
    )
    parser.add_argument(
        "-c",
        "--config",
        required=False,
        help="This file can be a YAML file, JSON file or JSON string",
    )

    # Specific args of each building block
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument(
        "--input_topology_path",
        required=True,
        help="Path to the input ligand topologies as a zip file containing a list of itp files.",
    )
    required_args.add_argument(
        "--output_topology_path",
        required=True,
        help="Path to the merged ligand topology file",
    )

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    pmxmerge_ff(
        input_topology_path=args.input_topology_path,
        output_topology_path=args.output_topology_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()



"""Common functions for package biobb_pmx.pmx"""

import re
from typing import Iterable, Mapping, Optional, Union

MUTATION_DICT = {
    "ALA": "A",
    "ARG": "R",
    "ASN": "N",
    "ASP": "D",
    "ASPH": "B",
    "ASPP": "B",
    "ASH": "B",
    "CYS": "C",
    "CYS2": "C",
    "CYN": "C",
    "CYX": "CX",
    "CYM": "CM",
    "CYSH": "C",
    "GLU": "E",
    "GLUH": "J",
    "GLUP": "J",
    "GLH": "J",
    "GLN": "Q",
    "GLY": "G",
    "HIS": "H",
    "HIE": "X",
    "HISE": "X",
    "HSE": "X",
    "HIP": "Z",
    "HSP": "Z",
    "HISH": "Z",
    "HID": "H",
    "HSD": "H",
    "ILE": "I",
    "LEU": "L",
    "LYS": "K",
    "LYSH": "K",
    "LYP": "K",
    "LYN": "O",
    "LSN": "O",
    "MET": "M",
    "PHE": "F",
    "PRO": "P",
    "SER": "S",
    "SP1": "SP1",
    "SP2": "SP2",
    "THR": "T",
    "TRP": "W",
    "TYR": "Y",
    "VAL": "V",
    "A": "A",
    "T": "T",
    "C": "C",
    "G": "G",
    "U": "U",
}


def create_mutations_file(
    input_mutations_path: str,
    mutation_list: Union[str, Iterable[str]],
    mutation_dict: Mapping,
) -> str:
    try:
        # Check if self.mutation_list is a string
        mutation_list = mutation_list.replace(" ", "").split(",")  # type: ignore
    except AttributeError:
        pass
    pattern = re.compile(r"(?P<chain>[a-zA-Z])*:?(?P<resnum>\d+)(?P<mt>[a-zA-Z0-9]+)")
    with open(input_mutations_path, "w") as mut_file:
        for mut in mutation_list:
            match = pattern.match(mut.strip())
            if match:
                mut_groups_dict = match.groupdict()
                if mut_groups_dict.get("chain"):
                    mut_file.write(mut_groups_dict.get("chain", "") + " ")
                mut_file.write(mut_groups_dict.get("resnum", "") + " ")
                if mut_groups_dict.get("mt", "").upper() not in mutation_dict.keys():
                    raise TypeError(
                        f"{mut_groups_dict.get('mt','').upper()} is not a valid AA code or NA code. Possible values are {mutation_dict.keys()}"
                    )
                mut_file.write(mutation_dict[mut_groups_dict.get("mt", "").upper()])
                mut_file.write("\n")

    return input_mutations_path


# TODO: Move this function to biobb_common.tools.file_utils
def _from_string_to_list(input_data: Optional[Union[str, list[str]]]) -> list[str]:
    """
    Converts a string to a list, splitting by commas or spaces. If the input is already a list, returns it as is.
    Returns an empty list if input_data is None.

    Parameters:
        input_data (str, list, or None): The string, list, or None value to convert.

    Returns:
        list: A list of string elements or an empty list if input_data is None.
    """
    if input_data is None:
        return []

    if isinstance(input_data, list):
        # If input is already a list, return it
        return input_data

    # If input is a string, determine the delimiter based on presence of commas
    delimiter = "," if "," in input_data else " "
    items = input_data.split(delimiter)

    # Remove whitespace from each item and ignore empty strings
    processed_items = [item.strip() for item in items if item.strip()]

    return processed_items
